# -*- coding:utf-8 -*-
from flask import request
from huansi_utils.webapi import ApiController

from app import api
from app.upgrade.upgrade_service import UpgradeService


@api('upgrade')
class UpgradeAPI(ApiController):
    '''
    1、回滚升级
    2、远程升级
    3、导出安装包
    4、导入安装包升级(不紧急)
    '''
    @api('apply')
    def get_apply_db_setting(self):
        '''
        应用数据库配置
        :return:
        '''
        return UpgradeService().apply_db_setting()

    @api('rollback/<int:log_id>')
    def get_rollback_upgrade(self, log_id):
        '''
        回滚升级
        :return:
        '''
        return UpgradeService().rollback_upgrade(log_id)

    @api('remote')
    def get_remote_upgrade(self):
        '''
        远程升级
        :return:
        '''
        log_id = request.args.get('log_id')
        return UpgradeService().remote_upgrade(log_id)

    @api('remote/step1')
    def get_remote_upgrade1(self):
        '''
        远程升级---1
        :return:
        '''
        return UpgradeService().remote_upgrade_step1()

    @api('remote/step2')
    def get_remote_upgrade2(self):
        '''
        远程升级---2
        :return:
        '''
        log_id = request.args.get('log_id')
        return UpgradeService().remote_upgrade_step2(log_id)

    @api('remote/step3/<int:app_count>/')
    def get_remote_upgrade3(self, app_count):
        '''
        远程升级---3
        :return:
        '''
        log_id = request.args.get('log_id')
        return UpgradeService().remote_upgrade_step3(app_count, log_id)

    @api('remote/step4')
    def get_remote_upgrade4(self):
        '''
        远程升级---4
        :return:
        '''
        log_id = request.args.get('log_id')
        return UpgradeService().remote_upgrade_step4(log_id)

    @api('remote/step5/<int:tar_file_index>')
    def get_remote_upgrade5(self, tar_file_index):
        '''
        远程升级---5
        :return:
        '''
        log_id = request.args.get('log_id')
        return UpgradeService().remote_upgrade_step5(tar_file_index, log_id)

    @api('remote/step6')
    def get_remote_upgrade6(self):
        '''
        远程升级---6
        :return:
        '''
        return UpgradeService().remote_upgrade_step6()

    @api('package_images/<int:log_id>')
    def get_package_images(self, log_id):
        '''
        导出镜像
        :param log_id:
        :return:
        '''
        return UpgradeService().package_images(log_id)
